@@ -22,7 +22,6 @@
 
 #define TEXT_TAG install("corpus::text")
 
-
 enum source_type {
 	SOURCE_NONE = 0,
 	SOURCE_CHAR,
@@ -40,9 +39,6 @@ struct source {
 };
 
 
-static void load_text(SEXP x);
-
-
 static int is_source(SEXP x)
 {
 	return (x == R_NilValue || TYPEOF(x) == STRSXP || is_json(x));
@@ -71,19 +67,46 @@ static void source_assign(struct source *source, SEXP value)
 
 static void free_text(SEXP stext)
 {
-        struct corpus_text *text = R_ExternalPtrAddr(stext);
-        free(text);
+        struct rcorpus_text *obj = R_ExternalPtrAddr(stext);
+	R_SetExternalPtrAddr(stext, NULL);
+
+	if (obj) {
+		if (obj->has_sentfilter) {
+			corpus_sentfilter_destroy(&obj->sentfilter);
+		}
+
+		if (obj->has_filter) {
+			corpus_filter_destroy(&obj->filter);
+		}
+
+		corpus_free(obj->text);
+		corpus_free(obj);
+	}
 }
 
 
-SEXP alloc_text(SEXP sources, SEXP source, SEXP row, SEXP start, SEXP stop)
+static void load_text(SEXP x);
+
+
+SEXP alloc_text_handle(void)
+{
+	SEXP ans;
+
+	PROTECT(ans = R_MakeExternalPtr(NULL, TEXT_TAG, R_NilValue));
+	R_RegisterCFinalizerEx(ans, free_text, TRUE);
+	UNPROTECT(1);
+	return ans;
+}
+
+
+SEXP alloc_text(SEXP sources, SEXP source, SEXP row, SEXP start, SEXP stop,
+		SEXP eltnames, SEXP filter)
 {
 	SEXP ans, handle, names, sclass, src, row_names, table;
 	R_xlen_t n;
 	int s, nsrc;
 
-	PROTECT(handle = R_MakeExternalPtr(NULL, TEXT_TAG, R_NilValue));
-	R_RegisterCFinalizerEx(handle, free_text, TRUE);
+	PROTECT(handle = alloc_text_handle());
 
 	n = XLENGTH(source);
 
@@ -100,6 +123,11 @@ SEXP alloc_text(SEXP sources, SEXP source, SEXP row, SEXP start, SEXP stop)
 	} else if (XLENGTH(stop) != n || TYPEOF(stop) != INTSXP) {
 		error("invalid 'stop' argument");
 	}
+	if (eltnames != R_NilValue) {
+		if (XLENGTH(eltnames) != n || TYPEOF(eltnames) != STRSXP) {
+			error("invalid 'names' argument");
+		}
+	}
 
 	nsrc = (int)XLENGTH(sources);
 	for (s = 0; s < nsrc; s++) {
@@ -132,17 +160,19 @@ SEXP alloc_text(SEXP sources, SEXP source, SEXP row, SEXP start, SEXP stop)
         SET_STRING_ELT(sclass, 0, mkChar("data.frame"));
         setAttrib(table, R_ClassSymbol, sclass);
 
-	PROTECT(ans = allocVector(VECSXP, 4));
+	PROTECT(ans = allocVector(VECSXP, 5));
 	SET_VECTOR_ELT(ans, 0, handle);
 	SET_VECTOR_ELT(ans, 1, sources);
 	SET_VECTOR_ELT(ans, 2, table);
-	SET_VECTOR_ELT(ans, 3, R_NilValue);
+	SET_VECTOR_ELT(ans, 3, eltnames);
+	SET_VECTOR_ELT(ans, 4, filter);
 
-	PROTECT(names = allocVector(STRSXP, 4));
+	PROTECT(names = allocVector(STRSXP, 5));
         SET_STRING_ELT(names, 0, mkChar("handle"));
         SET_STRING_ELT(names, 1, mkChar("sources"));
         SET_STRING_ELT(names, 2, mkChar("table"));
         SET_STRING_ELT(names, 3, mkChar("names"));
+        SET_STRING_ELT(names, 4, mkChar("filter"));
         setAttrib(ans, R_NamesSymbol, names);
 
 	PROTECT(sclass = allocVector(STRSXP, 1));
@@ -172,104 +202,113 @@ int is_text(SEXP x)
 }
 
 
+SEXP text_valid(SEXP x)
+{
+	return ScalarLogical(is_text(x));
+}
+
+
 struct corpus_text *as_text(SEXP stext, R_xlen_t *lenptr)
 {
-	SEXP handle, source, table;
-	struct corpus_text *text;
+	SEXP handle;
+	struct rcorpus_text *obj;
 
 	if (!is_text(stext)) {
 		error("invalid 'text' object");
 	}
 
 	handle = getListElement(stext, "handle");
-	text = R_ExternalPtrAddr(handle);
-	if (!text) {
+	obj = R_ExternalPtrAddr(handle);
+	if (!obj) {
 		load_text(stext);
 		handle = getListElement(stext, "handle");
-		text = R_ExternalPtrAddr(handle);
+		obj = R_ExternalPtrAddr(handle);
 	}
 
 	if (lenptr) {
-		table = getListElement(stext, "table");
-		if (table == R_NilValue) {
-			*lenptr = 0;
-		} else {
-			source = getListElement(table, "source");
-			if (source == R_NilValue) {
-				*lenptr = 0;
-			} else {
-				*lenptr = XLENGTH(source);
-			}
-		}
+		*lenptr = obj->length;
 	}
 
-	return text;
+	return obj->text;
 }
 
 
-SEXP as_text_json(SEXP sdata)
+SEXP as_text_json(SEXP sdata, SEXP filter)
 {
-	SEXP ans, handle, sources, source, row, start, stop;
+	SEXP ans, handle, sources, source, row, start, stop, names;
 	const struct json *d = as_json(sdata);
-	struct corpus_text *text;
+	struct rcorpus_text *obj;
 	R_xlen_t i, nrow = d->nrow;
-	int err;
+	int err = 0, nprot = 0;
 
-	PROTECT(sources = allocVector(VECSXP, 1));
+	PROTECT(sources = allocVector(VECSXP, 1)); nprot++;
 	SET_VECTOR_ELT(sources, 0, sdata);
 
-	PROTECT(source = allocVector(INTSXP, nrow));
+	PROTECT(source = allocVector(INTSXP, nrow)); nprot++;
 	for (i = 0; i < nrow; i++) {
+		RCORPUS_CHECK_INTERRUPT(i);
 		INTEGER(source)[i] = 1;
 	}
 
-	PROTECT(row = allocVector(REALSXP, nrow));
+	PROTECT(row = allocVector(REALSXP, nrow)); nprot++;
 	for (i = 0; i < nrow; i++) {
+		RCORPUS_CHECK_INTERRUPT(i);
 		REAL(row)[i] = (double)(i + 1);
 	}
 
-	PROTECT(start = allocVector(INTSXP, nrow));
-	PROTECT(stop = allocVector(INTSXP, nrow));
-	PROTECT(ans = alloc_text(sources, source, row, start, stop));
+	PROTECT(start = allocVector(INTSXP, nrow)); nprot++;
+	PROTECT(stop = allocVector(INTSXP, nrow)); nprot++;
+
+	names = R_NilValue;
+
+	PROTECT(ans = alloc_text(sources, source, row, start, stop, names,
+				 filter));
+	nprot++;
+
 	handle = getListElement(ans, "handle");
 
-	text = calloc(nrow, sizeof(*text));
-	if (nrow > 0 && !text) {
-		error("failed allocating memory (%"PRIu64" objects"
-		      " of size %u bytes)", (uint64_t)nrow, sizeof(*text));
+	TRY_ALLOC(obj = corpus_calloc(1, sizeof(*obj)));
+	R_SetExternalPtrAddr(handle, obj);
+
+	if (nrow > 0) {
+		TRY_ALLOC(obj->text = corpus_calloc(nrow, sizeof(*obj->text)));
+		obj->length = nrow;
 	}
-	R_SetExternalPtrAddr(handle, text);
 
 	for (i = 0; i < nrow; i++) {
-		if ((err = corpus_data_text(&d->rows[i], &text[i]))) {
-			text[i].ptr = NULL;
-			text[i].attr = 0;
+		RCORPUS_CHECK_INTERRUPT(i);
+		if ((err = corpus_data_text(&d->rows[i], &obj->text[i]))) {
+			obj->text[i].ptr = NULL;
+			obj->text[i].attr = 0;
 			INTEGER(start)[i] = NA_INTEGER;
 			INTEGER(stop)[i] = NA_INTEGER;
 		} else {
-			if (CORPUS_TEXT_SIZE(&text[i]) > INT_MAX) {
+			if (CORPUS_TEXT_SIZE(&obj->text[i]) > INT_MAX) {
 				error("text size (%"PRIu64 "bytes)"
 				      "exceeds maximum (%d bytes)",
-				      (uint64_t)CORPUS_TEXT_SIZE(&text[i]),
+				      (uint64_t)CORPUS_TEXT_SIZE(&obj->text[i]),
 				      INT_MAX);
 			}
 			INTEGER(start)[i] = 1;
-			INTEGER(stop)[i] = (int)CORPUS_TEXT_SIZE(&text[i]);
+			INTEGER(stop)[i] = (int)CORPUS_TEXT_SIZE(&obj->text[i]);
 		}
 	}
 
-	UNPROTECT(6);
+out:
+	UNPROTECT(nprot);
+	CHECK_ERROR(err);
 	return ans;
 }
 
 
-SEXP as_text_character(SEXP x)
+SEXP as_text_character(SEXP x, SEXP filter)
 {
-	SEXP ans, handle, sources, source, row, start, stop, str;
-	struct corpus_text *text;
+	SEXP ans, handle, sources, source, row, start, stop, names, str;
+	struct rcorpus_text *obj;
 	const char *ptr;
 	R_xlen_t i, nrow, len;
-	int err, iname, duped = 0;
+	int duped = 0;
+	int err = 0, nprot = 0;
 
 	if (x == R_NilValue || TYPEOF(x) != STRSXP) {
 	       error("invalid 'character' object");
@@ -282,41 +321,47 @@ SEXP as_text_character(SEXP x)
 		      (uint64_t)nrow, ((uint64_t)1) << DBL_MANT_DIG);
 	}
 
-	PROTECT(sources = allocVector(VECSXP, 1));
+	PROTECT(sources = allocVector(VECSXP, 1)); nprot++;
 	SET_VECTOR_ELT(sources, 0, x);
 
-	PROTECT(source = allocVector(INTSXP, nrow));
+	PROTECT(source = allocVector(INTSXP, nrow)); nprot++;
 	for (i = 0; i < nrow; i++) {
+		RCORPUS_CHECK_INTERRUPT(i);
 		INTEGER(source)[i] = 1;
 	}
 
-	PROTECT(row = allocVector(REALSXP, nrow));
+	PROTECT(row = allocVector(REALSXP, nrow)); nprot++;
 	for (i = 0; i < nrow; i++) {
+		RCORPUS_CHECK_INTERRUPT(i);
 		REAL(row)[i] = (double)(i + 1);
 	}
 
-	PROTECT(start = allocVector(INTSXP, nrow));
-	PROTECT(stop = allocVector(INTSXP, nrow));
-	PROTECT(ans = alloc_text(sources, source, row, start, stop));
+	PROTECT(start = allocVector(INTSXP, nrow)); nprot++;
+	PROTECT(stop = allocVector(INTSXP, nrow)); nprot++;
+	names = R_NilValue;
+
+	PROTECT(ans = alloc_text(sources, source, row, start, stop, names,
+				 filter));
+	nprot++;
+
 	handle = getListElement(ans, "handle");
 
-	text = calloc(nrow, sizeof(*text));
-	if (nrow > 0 && !text) {
-		error("failed allocating memory (%"PRIu64" objects"
-		      " of size %u bytes)", (uint64_t)nrow, sizeof(*text));
-	}
-	R_SetExternalPtrAddr(handle, text);
+	TRY_ALLOC(obj = corpus_calloc(1, sizeof(*obj)));
+	R_SetExternalPtrAddr(handle, obj);
 
-	iname = findListElement(ans, "names");
-	SET_VECTOR_ELT(ans, iname, getAttrib(x, R_NamesSymbol));
+	if (nrow > 0) {
+		TRY_ALLOC(obj->text = corpus_calloc(nrow, sizeof(*obj->text)));
+		obj->length = nrow;
+	}
 
 	for (i = 0; i < nrow; i++) {
+		RCORPUS_CHECK_INTERRUPT(i);
 		str = STRING_ELT(x, i);
 
 		// handle NA
 		if (str == NA_STRING) {
-			text[i].ptr = NULL;
-			text[i].attr = 0;
+			obj->text[i].ptr = NULL;
+			obj->text[i].attr = 0;
 			INTEGER(start)[i] = NA_INTEGER;
 			INTEGER(stop)[i] = NA_INTEGER;
 			continue;
@@ -349,7 +394,7 @@ SEXP as_text_character(SEXP x)
 			      " exceeds maximum (%d bytes)",
 			      (uint64_t)(i + 1), (uint64_t)len, INT_MAX);
 		}
-		if ((err = corpus_text_assign(&text[i], (uint8_t *)ptr,
+		if ((err = corpus_text_assign(&obj->text[i], (uint8_t *)ptr,
 					      (size_t)len,
 					      CORPUS_TEXT_NOESCAPE))) {
 			error("character object at index %"PRIu64
@@ -357,20 +402,16 @@ SEXP as_text_character(SEXP x)
 		}
 
 		INTEGER(start)[i] = 1;
-		INTEGER(stop)[i] = (int)CORPUS_TEXT_SIZE(&text[i]);
+		INTEGER(stop)[i] = (int)CORPUS_TEXT_SIZE(&obj->text[i]);
 	}
 
-	UNPROTECT(6);
+out:
+	UNPROTECT(nprot);
+	CHECK_ERROR(err);
 	return ans;
 }
 
 
-SEXP alloc_na_text(void)
-{
-	return as_text_character(ScalarString(NA_STRING));
-}
-
-
 SEXP coerce_text(SEXP sx)
 {
 	SEXP ans;
@@ -378,13 +419,12 @@ SEXP coerce_text(SEXP sx)
 	if (is_text(sx)) {
 		return sx;
 	} else if (is_json(sx)) {
-		return as_text_json(sx);
+		return as_text_json(sx, R_NilValue);
 	}
 
 	PROTECT(sx = coerceVector(sx, STRSXP));
-	ans = as_text_character(sx);
+	ans = as_text_character(sx, R_NilValue);
 	UNPROTECT(1);
-
 	return ans;
 }
 
@@ -394,18 +434,18 @@ static void load_text(SEXP x)
 	SEXP shandle, srow, ssource, sstart, sstop, ssources, src, str, stable;
 	const double *row;
 	const int *source, *start, *stop;
-	struct corpus_text *text;
+	struct rcorpus_text *obj;
 	struct corpus_text txt;
 	struct source *sources;
 	const uint8_t *ptr;
 	double r;
 	R_xlen_t i, j, len, nrow;
-	int s, nsrc, err, begin, end, flags;
+	int err = 0, s, nsrc, begin, end, flags = 0;
 
 	shandle = getListElement(x, "handle");
 
-	text = R_ExternalPtrAddr(shandle);
-	if (text) {
+	obj = R_ExternalPtrAddr(shandle);
+	if (obj) {
 		return;
 	}
 
@@ -447,14 +487,18 @@ static void load_text(SEXP x)
 	start = INTEGER(sstart);
 	stop = INTEGER(sstop);
 
-	text = calloc(nrow, sizeof(*text));
-	if (nrow > 0 && !text) {
-		error("failed allocating memory (%"PRIu64" objects"
-		      " of size %u bytes)", (uint64_t)nrow, sizeof(*text));
+	R_RegisterCFinalizerEx(shandle, free_text, TRUE);
+	TRY_ALLOC(obj = corpus_calloc(1, sizeof(*obj)));
+	R_SetExternalPtrAddr(shandle, obj);
+
+	if (nrow > 0) {
+		TRY_ALLOC(obj->text = corpus_calloc(nrow, sizeof(*obj->text)));
+		obj->length = nrow;
 	}
-	R_SetExternalPtrAddr(shandle, text);
 
 	for (i = 0; i < nrow; i++) {
+		RCORPUS_CHECK_INTERRUPT(i);
+
 		s = source[i];
 		if (!(1 <= s && s <= nsrc)) {
 			error("source[[%"PRIu64"]] (%d) is out of range",
@@ -469,18 +513,30 @@ static void load_text(SEXP x)
 		}
 		j = (R_xlen_t)(r - 1);
 
+		// handle NA range
+		if (start[i] == NA_INTEGER || stop[i] == NA_INTEGER) {
+			obj->text[i].ptr = NULL;
+			obj->text[i].attr = 0;
+			continue;
+		}
+
 		switch (sources[s].type) {
 		case SOURCE_CHAR:
 			str = STRING_ELT(sources[s].data.chars, j);
-			ptr = (const uint8_t *)CHAR(str);
-			len = XLENGTH(str);
-			flags = CORPUS_TEXT_NOESCAPE;
-			err = corpus_text_assign(&txt, ptr, len, flags);
-			if (err) {
-				error("character object in source %d"
-				      " at index %"PRIu64
-				      " contains invalid UTF-8",
-				      s + 1, (uint64_t)(j + 1));
+			if (str == NA_STRING) {
+				txt.ptr = NULL;
+				txt.attr = 0;
+			} else {
+				ptr = (const uint8_t *)CHAR(str);
+				len = XLENGTH(str);
+				flags = CORPUS_TEXT_NOESCAPE;
+				err = corpus_text_assign(&txt, ptr, len, flags);
+				if (err) {
+					error("character object in source %d"
+					      " at index %"PRIu64
+					      " contains invalid UTF-8",
+					      s + 1, (uint64_t)(j + 1));
+				}
 			}
 			break;
 
@@ -505,44 +561,14 @@ static void load_text(SEXP x)
 
 		// this could be made more efficient; add a
 		// 'can_break?' function to corpus/text.h
-		err = corpus_text_assign(&text[i], txt.ptr + begin,
+		err = corpus_text_assign(&obj->text[i], txt.ptr + begin,
 					 end - begin, flags);
-
 		if (err) {
 			error("text span in row[[%"PRIu64"]]"
 			      " starts or ends in the middle"
 			      " of a multi-byte character", i + 1);
 		}
 	}
-}
-
-
-SEXP subset_text_handle(SEXP handle, SEXP si)
-{
-	SEXP ans;
-	const struct corpus_text *text = R_ExternalPtrAddr(handle);
-	struct corpus_text *sub;
-	R_xlen_t i, n;
-	double ix;
-
-	PROTECT(ans = R_MakeExternalPtr(NULL, TEXT_TAG, R_NilValue));
-	R_RegisterCFinalizerEx(ans, free_text, TRUE);
-
-	if (text) {
-		n = XLENGTH(si);
-		sub = calloc(n, sizeof(*sub));
-		if (n && !sub) {
-			error("failed allocating %"PRIu64" bytes",
-			      (uint64_t)n * sizeof(*sub));
-		}
-		R_SetExternalPtrAddr(ans, sub);
-
-		for (i = 0; i < n; i++) {
-			ix = REAL(si)[i] - 1;
-			sub[i] = text[(R_xlen_t)ix];
-		}
-	}
-
-	UNPROTECT(1);
-	return ans;
+out:
+	CHECK_ERROR(err);
 }
