@@ -2,8 +2,10 @@ context("text_names")
 
 
 test_that("`names` should be NULL for new text", {
-    x <- as_text("ABC")
+    x <- as_text(c("A", "B", "C"))
     expect_equal(names(x), NULL)
+
+    expect_equal(names(as_text(character())), NULL)
 })
 
 
@@ -14,6 +16,13 @@ test_that("`names<-` should work on text", {
 })
 
 
+test_that("setting `names<-` to NULL should restore defaults", {
+    x <- as_text(c(a="x", b="y"))
+    names(x) <- NULL
+    expect_equal(names(x), NULL)
+})
+
+
 test_that("`as_text` should not drop names", {
     x <- as_text(c(a="1", b="2"))
     expect_equal(names(x), c("a", "b"))
@@ -67,7 +76,6 @@ test_that("`names<-` should not modify copies", {
 })
 
 
-
 test_that("`names<-` should preserve attributes", {
     x <- as_text(1:3)
     attr(x, "foo") <- "bar"
@@ -77,15 +85,20 @@ test_that("`names<-` should preserve attributes", {
 })
 
 
-test_that("`names<-` should allow NA", {
+test_that("`names<-` should not allow NA", {
     x <- as_text(1:3)
-    names(x) <- c("a", NA, "b")
-    expect_equal(names(x), c("a", NA, "b"))
+    expect_error(names(x) <- c("a", NA, "b"),
+                 "missing values in 'names' are not allowed")
 })
 
 
-test_that("`names<-` should allow duplicates", {
+test_that("`names<-` should not allow duplicates", {
     x <- as_text(1:3)
-    names(x) <- c("a", "b", "a")
-    expect_equal(names(x), c("a", "b", "a"))
+    expect_error(names(x) <- c("a", "b", "a"),
+                 "duplicate 'names' are not allowed")
+})
+
+
+test_that("names should error for non-text", {
+    expect_error(names.corpus_text("hello"), "invalid text object")
 })
